/*
 * setvoodoo.c, (c) 2002 John Clemens
 * Distributed under the terms and conditions of the GNU GPL.
 *
 * Command line utility to set the core and memory speeds of Voodoo cards.
 */

#include <stdio.h>
#include <voodoo.h>
#include <unistd.h>

#define _GNU_SOURCE
#include <getopt.h>

void print_help(void);

int main(int argc, char **argv) 
{
	struct voodoo_info *v, *t1;
	int new_gfxspeed, new_memspeed, card;
	int c, stop, i;
	
	card = 1;
	new_gfxspeed = 0;
	new_memspeed = 0;
	stop = 0;
	
	/* parse out the options */
	while (1) {
		int index = 0;
		static struct option long_options[] = {
			{"help", 0, 0, 'h'},
			{"gfxspeed", 1, 0, 'g'},
			{"memspeed", 1, 0, 'm'},
			{"card", 1, 0, 'c'},
			{0, 0, 0, 0}
		};
	
		c = getopt_long(argc, argv, "hg:m:c:", long_options, &index);
		if (c == -1) 
			break;

		switch (c) {
			case 'h':
				print_help();
				stop = 1;
				break;
			case 'c':
				card = strtol(optarg, (char **)NULL, 10);
				break;
			case 'g':
				new_gfxspeed = strtol(optarg, 
							(char **)NULL, 10);
				break;
			case 'm':
				new_memspeed = strtol(optarg, 
							(char **)NULL, 10);
				break;
		}
	}

	if (stop) 
		exit(0);

	/* Initialize libvoodoo */
	v = voodoo_probe();

	if (!v) {
		printf("No Voodoo cards found\n");
		exit(1);
	}
	
	i = 0;
	t1 = v;
	while (i < (card - 1)) {
		t1 = t1->next;
		i++;
	}

	if (!t1) {
		printf("Invalid card specified\n");
		exit(1);
	}
	
	printf("Found Voodoo: %s\n",t1->name);
	printf("MSize = %dMB MMIO = 0x%08x core = %dMhz mem = %dMhz\n",
			t1->memory_size >> 21, t1->mmio_addr, t1->gfx_speed,
			t1->memory_speed);
	
	if (new_gfxspeed || new_memspeed) {
		if (new_gfxspeed)
			printf("Setting Core speed to   : %d Mhz\n", 
					new_gfxspeed);
		if (new_memspeed)
			printf("Setting Memory speed to : %d Mhz\n", 
					new_memspeed);

		printf("Setting in 5 seconds, hit CTRL-C to abort!\n");
		sleep(5);

		if (new_gfxspeed) 
			voodoo_set_gfx_speed(t1, new_gfxspeed);
		if (new_memspeed)
			voodoo_set_mem_speed(t1, new_memspeed);
	}
	
	/* Clean up libvoodoo stuff */
	voodoo_cleanup(v);

	return 0;
}

/* Print out the help message */
void print_help(void)
{
	printf("setvoodoo, part of the Voodoo Utils, (c) 2002 John Clemens\n");
	printf("  Usage: setvoodoo <options>\n");
	printf("  Where options is one or more of:\n\n");
	printf("	-h --help	Prints this message\n");
	printf("	-c --card #	Which card to apply changes to\n");
	printf("			(numbers are the same as returned\n");
	printf("			 by 'listvoodoo')\n");
	printf("        -g --gfxspeed # Set core speed (in Mhz)\n");
	printf("	-m --memspeed # Set mem speed (in Mhz, Banshee only)");
	printf("\n\n");
	return;
}
